<?php

require_once(dirname(dirname(dirname(dirname(dirname(__FILE__))))) . '/oauth/libextinc/OAuth.php');

/**
 * Authenticate using OSM.
 *
 * @author Brook Schofield, TERENA.
 * @package simpleSAMLphp
 */
class sspmod_authosm_Auth_Source_OSM extends SimpleSAML_Auth_Source {

	/**
	 * The string used to identify our states.
	 */
	const STAGE_INIT = 'authosm:init';

	/**
	 * The key of the AuthId field in the state.
	 */
	const AUTHID = 'authosm:AuthId';

	private $key;
	private $secret;


	/**
	 * Constructor for this authentication source.
	 *
	 * @param array $info  Information about this authentication source.
	 * @param array $config  Configuration.
	 */
	public function __construct($info, $config) {
		assert('is_array($info)');
		assert('is_array($config)');

		/* Call the parent constructor first, as required by the interface. */
		parent::__construct($info, $config);

		if (!array_key_exists('key', $config))
			throw new Exception('OSM authentication source is not properly configured: missing [key]');

		$this->key = $config['key'];

		if (!array_key_exists('secret', $config))
			throw new Exception('OSM authentication source is not properly configured: missing [secret]');

		$this->secret = $config['secret'];
	}


	/**
	 * Log-in using OSM platform
	 * Documentation at: http://developer.OSM.com/docs/DOC-1008
	 *
	 * @param array &$state  Information about the current authentication.
	 */
	public function authenticate(&$state) {
		assert('is_array($state)');

		/* We are going to need the authId in order to retrieve this authentication source later. */
		$state[self::AUTHID] = $this->authId;

		$stateID = SimpleSAML_Auth_State::getStateId($state);
		SimpleSAML_Logger::debug('authosm auth state id = ' . $stateID);

		$consumer = new sspmod_oauth_Consumer($this->key, $this->secret);
//var_dump($consumer);
		// Get the request token
//		$requestToken = $consumer->getRequestToken('https://www.openstreetmap.org/oauth/request_token', array('oauth_callback' => SimpleSAML_Module::getModuleUrl('authosm') . '/linkback.php?stateid=' . $stateID));
$stateID=substr($stateID,0,strpos($stateID,':'));
$requestToken = $consumer->getRequestToken('https://www.openstreetmap.org/oauth/request_token', array('oauth_callback' => SimpleSAML_Module::getModuleUrl('authosm') . '/linkback.php?stateid=' . $stateID));
//var_dump($stateID);
//$requestToken->oauth_callback = $stateID;
//echo "<br><br>";
//var_dump(  $requestToken);
//echo"umiram";
//die;

		SimpleSAML_Logger::debug("Got a request token from the OAuth service provider [" .
			$requestToken->key . "] with the secret [" . $requestToken->secret . "]");

		$state['authosm:requestToken'] = $requestToken;

		// Update the state
		SimpleSAML_Auth_State::saveState($state, self::STAGE_INIT);

		// Authorize the request token
		$consumer->getAuthorizeRequest('https://www.openstreetmap.org/oauth/authorize', $requestToken);
	}


	public function finalStep(&$state) {
		$requestToken = $state['authosm:requestToken'];

		$consumer = new sspmod_oauth_Consumer($this->key, $this->secret);

		SimpleSAML_Logger::debug("oauth: Using this request token [" .
			$requestToken->key . "] with the secret [" . $requestToken->secret . "]");

		// Replace the request token with an access token (via GET method)
		$accessToken = $consumer->getAccessToken('https://www.openstreetmap.org/oauth/access_token', $requestToken,
			array('oauth_verifier' => $state['authosm:oauth_verifier']));

		SimpleSAML_Logger::debug("Got an access token from the OAuth service provider [" .
			$accessToken->key . "] with the secret [" . $accessToken->secret . "]");

		// TODO: configure attributes (http://developer.OSM.com/docs/DOC-1061) from config? Limited options via OSM.
//		$userdata = $consumer->getUserInfo('https://api.openstreetmap.org/api/0.6/user/details', $accessToken, array('http' => array('header' => 'x-li-format: xml')));
$userdata = $consumer->getUserInfo('https://api.openstreetmap.org/api/0.6/user/details', $accessToken,NULL,false);
//var_dump($userdata);die("Umiram");
$parser = xml_parser_create();
xml_parse_into_struct($parser,$userdata,$userdata);
xml_parser_free($parser);
$userdata= array(
    "id" => ($userdata[1]["attributes"]["ID"]),
    "nick" => ($userdata[1]["attributes"]["DISPLAY_NAME"]),
    "created" => ($userdata[1]["attributes"]["ACCOUNT_CREATED"]),
    "description" => ($userdata[2]["value"])
);
//var_dump($userdata);die("Umiram");
		$attributes = array();
		foreach($userdata AS $key => $value) {
			if (is_string($value))
				$attributes[$key] = array((string)$value);

		}

		// TODO: pass accessToken: key, secret + expiry as attributes?

//		if (array_key_exists('id', $userdata) ) {
//			$attributes['OSM_targetedID'] = array('http://OSM.com!' . $userdata['id']);
//			$attributes['OSM_user'] = array($userdata['id'] . '@OSM.com');
//		}
		$attributes['osm_user'] = array('Y');

		SimpleSAML_Logger::debug('OSM Returned Attributes: '. implode(", ",array_keys($attributes)));

		$state['Attributes'] = $attributes;
	}
}
